#ifndef CPPRUBY_EXCEPTION_HANDLER
#define CPPRUBY_EXCEPTION_HANDLER

#include <ruby.h>

namespace CppRuby
{

class Object;


/// Handles 'protection' failure
class ExceptionHandler
{
public:
  VALUE handleException();

private:
  /*! \brief Called when a Ruby exception is raised.
   * \param error Exception '$!'
   * \return value returned by protect().
   */
  virtual VALUE handleException( Object error ) = 0;
};


#ifdef AFX_API
/// Display dialog & trace to debug output
class MfcExceptionHandler : public ExceptionHandler
{
private:
  VALUE handleException( Object error );
};

#endif


#ifdef WIN32
/// Trace to debug output
class TraceExceptionHandler : public ExceptionHandler
{
private:
  VALUE handleException( Object error );
};
#endif


/// Trace to stderr output
class StdErrExceptionHandler : public ExceptionHandler
{
private:
  VALUE handleException( Object error );
};


#ifdef WIN32
  typedef TraceExceptionHandler DefaultExceptionHandler;
#else
  typedef StdErrExceptionHandler DefaultExceptionHandler;
#endif


}  // namespace CppRuby


#endif