// EditorView.cpp : implementation of the EditorView class
//

#include "stdafx.h"
#include "PraticalRuby.h"
#include "EditorView.h"
#include "MainFrame.h"

#include <Scintilla.h>
#include <SciLexer.h>
#include "RubyExtScintilla.h"
#include "RubyTools.h"
#include "CppRuby.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// EditorView

EditorView::EditorView()
    : m_rubyScintilla( Qnil )
{
}

EditorView::~EditorView()
{
}


BEGIN_MESSAGE_MAP(EditorView,CWnd )
  //{{AFX_MSG_MAP(EditorView)
  ON_WM_CREATE()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// EditorView message handlers

BOOL EditorView::PreCreateWindow(CREATESTRUCT& cs) 
{
  if (!CWnd::PreCreateWindow(cs))
    return FALSE;

//  cs.dwExStyle |= WS_EX_CLIENTEDGE;
//  cs.style &= ~WS_BORDER;
//  cs.lpszClass = AfxRegisterWndClass(CS_HREDRAW|CS_VREDRAW|CS_DBLCLKS, 
//    ::LoadCursor(NULL, IDC_ARROW), HBRUSH(COLOR_WINDOW+1), NULL);

  // Modify style to use Scintilla edit control...
  cs.lpszClass = "Scintilla";
  cs.style = WS_CHILD | WS_VSCROLL | WS_HSCROLL | WS_CLIPCHILDREN;


  return TRUE;
}

int EditorView::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
  if (CWnd ::OnCreate(lpCreateStruct) == -1)
    return -1;

  ShowWindow( SW_SHOW );
  SetFocus();

  createRubyScintilla();

  return 0;
}



LRESULT 
EditorView::SendEditor( UINT Msg, WPARAM wParam, LPARAM lParam )
{
  return SendMessage( Msg, wParam, lParam );
}



struct CreateRubyScintillaFunctor
{
  VALUE operator()( EditorView *editorWindow, 
                    CppRuby::Object mainFrame )
  {
    CppRuby::Klass classEditControl( "EditControl" );

    CppRuby::Object editorWindowHandle = UINT2NUM( (DWORD)editorWindow->GetSafeHwnd() );
    return classEditControl.newInstance( editorWindowHandle, mainFrame );
  }
};


void 
EditorView::createRubyScintilla()
{
  m_rubyScintilla = CppRuby::protect2( CreateRubyScintillaFunctor(), 
                                       this,
                                       MainFrame::instance().mainFrameObject(),
                                       CppRuby::MfcExceptionHandler() );
}

struct EditorViewFowardFunctor
{
  bool operator()( SCNotification *notification,
                   VALUE target )
  {
    Scintilla_Forward_Notification( notification, target );
    return true;
  }
};


// Notes that the notification message is sent to the parent window of the edit control,
// that is EditorFrame. EditorFrame forward notification to this window if it is the
// sender of the notification.
//
// This can be done with the following code in OnNotify of the frame:
//   LPNMHDR notification = (LPNMHDR)lParam;
//  if ( notification->hwndFrom == m_wndView.GetSafeHwnd() )
//    return m_wndView.SendMessage( WM_NOTIFY, wParam, lParam );

BOOL 
EditorView::OnNotify( WPARAM wParam, 
                      LPARAM lParam, 
                      LRESULT *pResult )
{
  SCNotification *notification = reinterpret_cast<SCNotification *>( lParam );
  if ( notification != NULL  &&  notification->nmhdr.hwndFrom == GetSafeHwnd() )
  {
    CppRuby::protect2( EditorViewFowardFunctor(), notification, m_rubyScintilla );
    return TRUE;
  }

  return CWnd ::OnNotify(wParam, lParam, pResult);
}


void 
EditorView::markGcCollectable( GcMarker &marker )
{
  marker.mark( m_rubyScintilla );
}


MainFrame &
EditorView::mainFrame()
{
  MainFrame *mainFrame = STATIC_DOWNCAST(MainFrame, AfxGetApp()->m_pMainWnd );
  return *mainFrame;
}


CppRuby::Object 
EditorView::editor()
{
  return m_rubyScintilla;
}
