#include "stdafx.h"

#include <algorithm>
#include <vector>
#include "GcCollectable.h"
#include "RubyTools.h"

// GcMarker implementation
// -----------------------

void 
GcMarker::mark( VALUE object )
{
  if ( object != Qnil )
  {
    // Should be: rb_gc_mark( object );
    // But the function is not declared correctly in hearders !!!

    // Tentative work-around:
    rb_gc_mark_locations( &object, &object );
  }
}



// GcCollectableOwnerManager declaration
// -------------------------------------

class GcCollectableOwnerManager
{
public:
  static bool initialize( const char **failureMessage );

  static GcCollectableOwnerManager &instance()
  {
    static GcCollectableOwnerManager instance;
    return instance;
  }

  void add( GcCollectableOwner *owner );

  void remove( GcCollectableOwner *owner );

  void markCollectables();

private:
  GcCollectableOwnerManager();

  typedef std::vector<GcCollectableOwner *> Owners;
  Owners m_owners;
};


// GcCollectableOwner implementation
// ---------------------------------



GcCollectableOwner::GcCollectableOwner()
{
  GcCollectableOwnerManager::instance().add( this );
}


GcCollectableOwner::~GcCollectableOwner()
{
  GcCollectableOwnerManager::instance().remove( this );
}




// GcCollectableOwnerManager implementation
// ----------------------------------------


GcCollectableOwnerManager::GcCollectableOwnerManager()
{
}


static void 
GcMarkObjects( void *dummyObject )
{
  GcCollectableOwnerManager::instance().markCollectables();
}



void Init_GcCollectableOwnerManager()
{
//  VALUE dummyObject = Data_Wrap_Struct( rb_cObject, GcMarkObjects, NULL, NULL );

//  rb_define_readonly_variable( "___PraticalRubyGcCollectableOwnerManager",
//                               &dummyObject );
}


void 
GcCollectableOwnerManager::add( GcCollectableOwner *owner )
{
  m_owners.push_back( owner );
}


void 
GcCollectableOwnerManager::remove( GcCollectableOwner *owner )
{
  Owners::iterator found = std::find( m_owners.begin(), m_owners.end(), owner );
  if ( found != m_owners.end() )
    m_owners.erase( found );
}


void 
GcCollectableOwnerManager::markCollectables()
{
  GcMarker marker;
  for ( Owners::iterator it = m_owners.begin(); it != m_owners.end(); ++it )
    (*it)->markGcCollectable( marker );
}
