// MainFrame.cpp : implementation of the MainFrame class
//

#include "stdafx.h"
#include "MainFrame.h"
#include "PraticalRuby.h"
#include "ActionStatusUpdater.h"
#include "EditorFrame.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// MainFrame

IMPLEMENT_DYNAMIC(MainFrame, CMDIFrameWnd)

BEGIN_MESSAGE_MAP(MainFrame, CMDIFrameWnd)
	//{{AFX_MSG_MAP(MainFrame)
	ON_WM_CREATE()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

static UINT indicators[] =
{
	ID_SEPARATOR,           // status line indicator
	ID_INDICATOR_CAPS,
	ID_INDICATOR_NUM,
	ID_INDICATOR_SCRL,
};

/////////////////////////////////////////////////////////////////////////////
// MainFrame construction/destruction


struct MainFrame::MainFrameInitFunctor
{
  VALUE operator()()
  {
    return CppRuby::Klass("MainFrame").newInstance();
  }
};


MainFrame::MainFrame()
    : m_mainFrame( CppRuby::protect( MainFrameInitFunctor(), 
                   CppRuby::MfcExceptionHandler() ) )
{
}


MainFrame::~MainFrame()
{
  //TODO: handle additional resources you may have added
  if ( m_hMDIMenu != NULL )
    ::FreeResource( m_hMDIMenu );
  if ( m_hMDIAccel != NULL )
    ::FreeResource( m_hMDIAccel );
}


int 
MainFrame::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
  // try to load shared MDI menus and accelerator table
  //TODO: add additional member variables and load calls for
  //	additional menu types your application may need. 

  HINSTANCE hInst = AfxGetResourceHandle();
  m_hMDIMenu  = ::LoadMenu(hInst, MAKEINTRESOURCE(IDR_PRATICTYPE));
  m_hMDIAccel = ::LoadAccelerators(hInst, MAKEINTRESOURCE(IDR_PRATICTYPE));

  if (CMDIFrameWnd::OnCreate(lpCreateStruct) == -1)
	  return -1;
  
  if (!m_wndToolBar.CreateEx(this, TBSTYLE_FLAT, WS_CHILD | WS_VISIBLE | CBRS_TOP
	  | CBRS_GRIPPER | CBRS_TOOLTIPS | CBRS_FLYBY | CBRS_SIZE_DYNAMIC) ||
	  !m_wndToolBar.LoadToolBar(IDR_MAINFRAME))
  {
	  TRACE0("Failed to create toolbar\n");
	  return -1;      // fail to create
  }

  if (!m_wndStatusBar.Create(this) ||
	  !m_wndStatusBar.SetIndicators(indicators,
	    sizeof(indicators)/sizeof(UINT)))
  {
	  TRACE0("Failed to create status bar\n");
	  return -1;      // fail to create
  }

  // TODO: Delete these three lines if you don't want the toolbar to
  //  be dockable
  m_wndToolBar.EnableDocking(CBRS_ALIGN_ANY);
  EnableDocking(CBRS_ALIGN_ANY);
  DockControlBar(&m_wndToolBar);



  return 0;
}

/////////////////////////////////////////////////////////////////////////////
// MainFrame diagnostics

#ifdef _DEBUG
void MainFrame::AssertValid() const
{
	CMDIFrameWnd::AssertValid();
}

void MainFrame::Dump(CDumpContext& dc) const
{
	CMDIFrameWnd::Dump(dc);
}

#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// MainFrame message handlers


BOOL 
MainFrame::OnCommand( WPARAM wParam, 
                      LPARAM lParam )
{
  bool fromControl = (lParam != 0);
  bool fromMenu = !fromControl  &&  ((wParam & 0xffff0000) == 0);
  bool fromAccelerator = !fromControl  &&  !fromMenu;
  int commandId = (wParam & 0xffff);

  if ( executeCommand( commandId ) != Qnil )
    return TRUE;

  return CMDIFrameWnd::OnCommand(wParam, lParam);
}


CppRuby::Object
MainFrame::executeCommand( unsigned int commandId )
{
  CppRuby::Object action = m_mainFrame.safeMethodCall( "actionFor", 
                                                       UINT2NUM( commandId ) );
  if ( !action.isNil() )
    return action.safeMethodCall( "execute", CppRuby::MfcExceptionHandler() );
  return Qnil;
}


// Called for update ui and command
// nCode: CN_UPDATE_COMMAND_UI => when updating UI, should return TRUE and
//        change the enabled state of the command.
// nID: command id, >= 0x8000
// pExtra: cast to CCmdUi*, if nCode is CN_UPDATE_COMMAND_UI => enable/check...
//         command.
BOOL 
MainFrame::OnCmdMsg( UINT nID, 
                     int nCode, 
                     void* pExtra, 
                     AFX_CMDHANDLERINFO* pHandlerInfo )
{
  if ( nCode == CN_UPDATE_COMMAND_UI )
  {
    CCmdUI *cmdUi = static_cast<CCmdUI *>( pExtra );

    if ( nID == ID_FIND_NEXT )
    {
      int z =3;
    }

    CppRuby::Object action = m_mainFrame.safeMethodCall( "actionFor", 
                                                         UINT2NUM( nID ) );
    if ( !action.isNil() )
    {
      CppRuby::Object updater = Wrap_ActionStatusUpdater( cmdUi );
      action.safeMethodCall( "updateStatus", updater, CppRuby::MfcExceptionHandler() );
      return TRUE;
    }
  }
	
  return CMDIFrameWnd::OnCmdMsg(nID, nCode, pExtra, pHandlerInfo);
}


void 
MainFrame::markGcCollectable( GcMarker &marker )
{
//  marker.mark( m_mainFrame );
}


CppRuby::Object 
MainFrame::mainFrameObject()
{
  return m_mainFrame;
//  return Qnil;
}


void 
MainFrame::windowActivated( CppRuby::Object window )
{
  m_mainFrame.safeMethodCall( "activeWindow=", window );
}


EditorFrame *
MainFrame::newEditorWindow()
{
  MainFrame* pFrame = STATIC_DOWNCAST(MainFrame, AfxGetApp()->m_pMainWnd);

  // create a new MDI child window
  EditorFrame *editor = static_cast<EditorFrame *>( pFrame->CreateNewChild(
	                    RUNTIME_CLASS(EditorFrame), 
                            IDR_PRATICTYPE, 
                            m_hMDIMenu, 
                            m_hMDIAccel) );
  return editor;
}


MainFrame &
MainFrame::instance()
{
  return *STATIC_DOWNCAST(MainFrame, AfxGetApp()->m_pMainWnd );
}


// Need to override this method because of bug in MFC implementation
// => does not test if the active child is maximized when not using document/view
//   which resulted in the title being displayed twice on the main frame when MDI
//   child window was maximized.
void 
MainFrame::OnUpdateFrameTitle( BOOL bAddToTitle )
{
  if ( (GetStyle() & FWS_ADDTOTITLE) == 0 )
	  return;     // leave it alone!

  CMDIChildWnd* pActiveChild = NULL;
  CDocument* pDocument = GetActiveDocument();
  if (bAddToTitle &&
    (pActiveChild = MDIGetActive()) != NULL &&
    (pActiveChild->GetStyle() & WS_MAXIMIZE) == 0 &&
    (pDocument != NULL ||
     (pDocument = pActiveChild->GetActiveDocument()) != NULL))
	  UpdateFrameTitleForDocument(pDocument->GetTitle());
  else
  {
    CString strTitle;
    LPCTSTR title = NULL;

    if ( pActiveChild != NULL  &&  (pActiveChild->GetStyle() & WS_MAXIMIZE) == 0 )
    {
      strTitle = pActiveChild->GetTitle();
      title = strTitle;
    }
    UpdateFrameTitleForDocument( title );
  }
}
