// PraticalRuby.cpp : Defines the class behaviors for the application.
//

#include "stdafx.h"
#include "PraticalRuby.h"
#include "MainFrame.h"
#include "AboutDialog.h"

#include <ruby.h>
#include "RubyExtDialog.h"
#include "RubyExtEditControl.h"
#include "RubyExtFileDialog.h"
#include "RubyExtMainFrame.h"
#include "RubyExtScintilla.h"
#include "RubyExtTraceDebug.h"
#include "RubyTools.h"
#include "GcCollectable.h"
#include "ActionStatusUpdater.h"
#include "RubyCppTools.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// PraticalRubyApp

BEGIN_MESSAGE_MAP(PraticalRubyApp, CWinApp)
  //{{AFX_MSG_MAP(PraticalRubyApp)
  ON_COMMAND(ID_APP_ABOUT, OnAppAbout)
	  // NOTE - the ClassWizard will add and remove mapping macros here.
	  //    DO NOT EDIT what you see in these blocks of generated code!
  //}}AFX_MSG_MAP
  // Standard file based document commands
  // Standard print setup command
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// PraticalRubyApp construction

PraticalRubyApp::PraticalRubyApp()
{
}

/////////////////////////////////////////////////////////////////////////////
// The one and only PraticalRubyApp object

PraticalRubyApp theApp;

/////////////////////////////////////////////////////////////////////////////
// PraticalRubyApp initialization

BOOL PraticalRubyApp::InitInstance()
{
  AfxEnableControlContainer();

  // Standard initialization
  // If you are not using these features and wish to reduce the size
  //  of your final executable, you should remove from the following
  //  the specific initialization routines you do not need.

#ifdef _AFXDLL
  Enable3dControls();			// Call this when using MFC in a shared DLL
#else
  Enable3dControlsStatic();	// Call this when linking to MFC statically
#endif

  // Change the registry key under which our settings are stored.
  // TODO: You should modify this string to be something appropriate
  // such as the name of your company or organization.
  SetRegistryKey(_T("Gaia Creation"));

  if ( !initializeScintilla()  ||
       !initializeRuby() )
    return FALSE;

  LoadStdProfileSettings(9);  // Load standard INI file options (including MRU)

  // To create the main window, this code creates a new frame window
  // object and then sets it as the application's main window object.

  CMDIFrameWnd* pFrame = new MainFrame;
  m_pMainWnd = pFrame;

  // create main MDI frame window
  if (!pFrame->LoadFrame(IDR_MAINFRAME))
	  return FALSE;

  // Parse command line for standard shell commands, DDE, file open
  CCommandLineInfo cmdInfo;
  ParseCommandLine(cmdInfo);

  // The main window has been initialized, so show and update it.
  pFrame->ShowWindow(m_nCmdShow);
  pFrame->UpdateWindow();

  // Dispatch commands specified on the command line
  if ( cmdInfo.m_nShellCommand == CCommandLineInfo::FileOpen )
  {
    if ( !openEditorFile( cmdInfo.m_strFileName ) )
      return FALSE;
  }
  else
    pFrame->SendMessage( WM_COMMAND, ID_FILE_NEW, 0 );

  return TRUE;
}


// App command to run the dialog
void 
PraticalRubyApp::OnAppAbout()
{
  AboutDialog aboutDlg;
  aboutDlg.DoModal();
}

/////////////////////////////////////////////////////////////////////////////
// PraticalRubyApp message handlers


int 
PraticalRubyApp::ExitInstance() 
{
  return CWinApp::ExitInstance();
}


bool 
PraticalRubyApp::initializeScintilla()
{
  if ( !::LoadLibrary("SciLexer.dll") )    // Scintilla edit control
  {
    ::AfxMessageBox( IDS_ERROR_LOADING_SCILEXER_DLL, MB_OK | MB_ICONERROR );
    return false;
  }
  return true;
}


struct PraticalRubyApp::RubyInitializationFunctor
{
  VALUE operator()()
  {
    CString scriptPath = __argv[0];
    scriptPath = scriptPath.Left( scriptPath.ReverseFind( '\\' ) );
    CString debug= "Debug";
    if ( scriptPath.Right( debug.GetLength() ) == debug )
      scriptPath = scriptPath.Left( scriptPath.ReverseFind( '\\' ) );
    scriptPath += "\\script";


    ruby_incpush( scriptPath );   // add ./script to script include dir...
    ruby_load( "PraticalRubyRequires.rb" );
    Init_TraceDebug();
    Init_Scintilla();
    Init_GcCollectableOwnerManager();
    Init_ActionStatusUpdater();
    Init_FileDialog();
    Init_MainFrame();
    Init_EditControl();
    Init_Dialog();
    return Qtrue;
  }
};

bool 
PraticalRubyApp::initializeRuby()
{
//  NtInitialize(&__argc, &__argv);       // Unknown symbol !!!

  ruby_init();

  return CppRuby::protect( RubyInitializationFunctor(), 
                           CppRuby::MfcExceptionHandler() ) == Qtrue;
}


BOOL 
PraticalRubyApp::SaveAllModified() 
{
  BOOL canExit = CWinApp::SaveAllModified();
// temporary, should be moved to ruby (but need dialog API)
  CppRuby::Object mainFrame = MainFrame::instance().mainFrameObject();
  if ( mainFrame.safeMethodCall( "hasDirtyEditor?" ) != Qfalse )
  {
    CString message = "Some windows have not been saved.\nExit anyway ?\n"
                      "If you answer OK, changes will be lost.";
    if ( ::AfxMessageBox( message, MB_OKCANCEL ) != IDOK )
      canExit = FALSE;
  }

  return canExit;
}


bool 
PraticalRubyApp::openEditorFile( const char *fileName )
{
  CppRuby::Object mainFrame = MainFrame::instance().mainFrameObject();
  CppRuby::Object editor = mainFrame.safeMethodCall( "newEditorWindow" );
  if ( editor == Qnil )
    return false;

  if ( editor.safeMethodCall( "loadFile", 
                              rb_str_new2( fileName ), 
                              CppRuby::MfcExceptionHandler() ) != Qtrue )
    return false;
  return true;
}
