#ifndef RUBY_CPP_TOOLS
#define RUBY_CPP_TOOLS

#include "RubyTools.h"
#include "CppRubyExceptionHandler.h"


namespace CppRuby
{

template<typename Functor>
struct FunctorCaller
{
  static VALUE callFunctor( Functor *f )
  {
    return (*f)();
  }
};


template<typename Functor>
VALUE 
protect( Functor &functor, 
         ExceptionHandler &handler = DefaultExceptionHandler() )
{
  int error = 0;
  VALUE result = ruby_protect( FunctorCaller<Functor>::callFunctor, 
                               (VALUE)&functor,
                               &error );
  if ( error )
    result = handler.handleException();

  return result;
}


template<typename Functor,
         typename TypeParam1>
struct FunctorCaller1
{
  FunctorCaller1( Functor &functor,
                  TypeParam1 &param1 ) 
     : m_param1( param1 )
     , m_functor( functor )
  {
  }

  VALUE operator()()
  {
    return m_functor( m_param1 );
  }

  Functor &m_functor;
  TypeParam1 &m_param1;
};


template<typename Functor, 
         typename TypeParam1>
VALUE 
protect1( Functor &functor, 
          TypeParam1 &param1, 
          ExceptionHandler &handler = DefaultExceptionHandler() )
{
  FunctorCaller1<Functor,TypeParam1> caller( functor, param1 );
  return protect( caller, handler );
}


template<typename Functor,
         typename TypeParam1,
         typename TypeParam2>
struct FunctorCaller2
{
  FunctorCaller2( Functor &functor,
                  TypeParam1 &param1,
                  TypeParam2 &param2 ) 
     : m_param1( param1 )
     , m_param2( param2 )
     , m_functor( functor )
  {
  }

  VALUE operator()()
  {
    return m_functor( m_param1, m_param2 );
  }

  Functor &m_functor;
  TypeParam1 &m_param1;
  TypeParam2 &m_param2;
};


template<typename Functor, 
         typename TypeParam1, 
         typename TypeParam2>
VALUE 
protect2( Functor &functor, 
          TypeParam1 &param1, 
          TypeParam2 &param2, 
          ExceptionHandler &handler = DefaultExceptionHandler() )
{
  FunctorCaller2<Functor,TypeParam1,TypeParam2> caller( functor, param1, param2 );
  return protect( caller, handler );
}


struct EvalFunctor
{
  VALUE operator()( const char *text )
  {
    return rb_eval_string( text );
  }
};


inline VALUE 
protectedEval( const char *text, 
               ExceptionHandler &handler = DefaultExceptionHandler() )
{
  return protect1( EvalFunctor(), text, handler );
}

}  // namespace CppRuby

#endif