#include "RubyTools.h"

extern "C" {

/*! Protected loads of a ruby script.
 */
bool ruby_safe_load( const char *scriptFileName,
                     const char **failureMessage )
{
  int loadFailed = 0;
  rb_load_protect( rb_str_new2( scriptFileName ), 0, &loadFailed );
  if ( loadFailed )
  {
    if ( failureMessage )
      *failureMessage = STR2CSTR( rb_inspect( rb_gv_get("$!") ) );
    return false;
  }
  return true;
}


void 
ruby_load( const char *scriptFileName )
{
  rb_load( rb_str_new2( scriptFileName ), 0 );
}


// VC++ doesn't do the automatic cast in rb_define_method
typedef VALUE (*RubyMethod)(...);

void ruby_define_method( VALUE value ,
                         const char *name,
                         void *function,
                         int argc )
{
  rb_define_method( value, name, (RubyMethod)(function), argc );
}


void ruby_define_module_function( VALUE classOrModule,
                                  const char *name,
                                  void *function,
                                  int argc )
{
  rb_define_module_function( classOrModule, name, (RubyMethod)(function), argc );
}


// VC++ doesn't do the automatic cast in rb_protect_method
typedef VALUE (*RubyProtectedCall)();


VALUE ruby_protect( void *function,
                    VALUE data,
                    int *error )
{
  return rb_protect( (RubyProtectedCall)function, data, error );
}


VALUE ruby_get_class( const char *className )
{
  return rb_const_get( rb_cObject, rb_intern( className ) );
}




}  // extern "C"
