#ifndef GAIACRTN_RUBY_TOOLS_H
#define GAIACRTN_RUBY_TOOLS_H

#include <ruby.h>

extern "C" {

  /*! Protected loads of a ruby script.
   */
  bool ruby_safe_load( const char *scriptFileName,
                       const char **failureMessage = NULL );

  /*! \brief Unprotected load.
   */
  void ruby_load( const char *scriptFileName );

  /*! Defines a ruby method.
   *
   * (Work around VC++ function cast)
   */
  void ruby_define_method( VALUE classOrModule,
                           const char *name,
                           void *function,
                           int argc );

  /*! Defines a static ruby function (class or module method)
   *
   * (Work around VC++ function cast)
   */
  void ruby_define_module_function( VALUE classOrModule,
                                    const char *name,
                                    void *function,
                                    int argc );

  VALUE ruby_protect( void *function,
                      VALUE data,
                      int *error );

  /*! \brief Gets an existing class object.
   */
  VALUE ruby_get_class( const char *className );

}



#endif // GAIACRTN_RUBY_TOOLS_H