require 'ScintillaBridge.rb'
require 'TraceDebug.rb'

class Array
  def indexNotOf( object )
    index =0
    while index < length()
      if at(index) != object
        break;
	  end
	  index = index+1
	end
    return index
  end
end



class EditControl < ScintillaBridge

  attr_reader :fileName, :title, :dirty

  def initialize( editWindowHandle, mainFrame )
    super( editWindowHandle )
		@mainFrame = mainFrame
    @fileName = nil
		@title = ''
		@bookmarks = Array.new
		@dirty = false
    setUpEditor()
    setUpIndent()
  end

  # predefined color (from Color, but don't seem to works)
  Black  = Color.new( 0x000000 )
  Silver = Color.new( 0xC0C0C0 )
  Gray   = Color.new( 0x808080 )
  White  = Color.new( 0xFFFFFF )
  Maroon = Color.new( 0x800000 )
  Red    = Color.new( 0xFF0000 )
  Purple = Color.new( 0x800080 )
  Fuchsia= Color.new( 0xFF00FF )
  Green  = Color.new( 0x008000 )
  Lime   = Color.new( 0x00FF00 )
  Olive  = Color.new( 0x808000 )
  Yellow = Color.new( 0xFFFF00 )
  Navy   = Color.new( 0x000080 )
  Blue   = Color.new( 0x0000FF )
  Teal   = Color.new( 0x008080 )
  Aqua   = Color.new( 0x00FFFF )
  AquaBlue = Color.new( 0x0080C0 )
  OffWhite = Color.new( 0xFFFBF0 )

  def setUpEditor

    # use RUBY lexer
		self.lexer = SCLEX_RUBY					# weird, it seems that 'lexer = SCLEX_RUBY' declare
																			# declare a local variable instead of calling lexer=()
								# => to investigate, using self. in the meanwhile

		# set RUBY keywords
		setKeyWords( 0, %q[
		__LINE__ __FILE__ BEGIN END alias and begin break case class def defined? do
		else elsif end ensure false for if in module next nil not or
		redo rescue retry return self super then true undef unless until
		when while yield' ] )

		# set a flashy color for all styles, so we can spot those we forgot to set...
		for style in 0..STYLE_LASTPREDEFINED
			setStyle( style, Yellow )
		end

		# setting up style colors
		setStyle( SCE_P_STRING, Red )           # "..."
		setStyle( SCE_P_CHARACTER, Purple )     # '...'

		setStyle( STYLE_DEFAULT, Black, White, 11, "Courier New" )
		setStyle( SCE_P_DEFAULT, Black, White, 11, "Courier New" )

		setStyle( SCE_P_WORD, Blue )            # for keywords: class def module...

		setStyle( SCE_P_CLASSNAME, Teal )       # name following def, class or module
		setStyle( SCE_P_DEFNAME, Navy )         # name following def, class or module

		setStyle( SCE_P_STRING, Red )           # "..."
		setStyle( SCE_P_CHARACTER, Purple )     # '...'

		setStyle( SCE_P_OPERATOR, AquaBlue )    # + - / = ...

		setStyle( SCE_P_NUMBER, Maroon )        # 123.456

		setStyle( SCE_P_TRIPLEDOUBLE, Green )   # =begin ... =end
		setStyle( SCE_P_COMMENTLINE, Green )    # # ...

		setStyle( SCE_P_IDENTIFIER, Black )     # identifiers

		setStyle( STYLE_BRACELIGHT, White, AquaBlue ) # brace highlight
		setStyle( STYLE_BRACEBAD, White, Red )	      # brace bad highlight

		setStyle( STYLE_LINENUMBER, Black, OffWhite ) # line number
		setStyle( STYLE_LINENUMBER, Black, OffWhite ) # line number

		@lineNumberMargin = 0
		@symbolMargin = 1
		@folderMargin = 2
		setMarginTypeN( @lineNumberMargin, SC_MARGIN_NUMBER )
		setMarginWidthN( @lineNumberMargin, 32 )

		setMarginTypeN( @symbolMargin, SC_MARGIN_SYMBOL )
		setMarginWidthN( @symbolMargin, 12 )

		setMarginTypeN( @folderMargin, SC_MARGIN_SYMBOL )
		setMarginWidthN( @folderMargin, 12 )
		setMarginMaskN( @folderMargin, SC_MASK_FOLDERS )
		setMarginSensitiveN( @folderMargin, true )

		markerDefine( SC_MARKNUM_FOLDEROPEN, SC_MARK_MINUS )
		markerDefine( SC_MARKNUM_FOLDER, SC_MARK_PLUS )
		markerDefine( SC_MARKNUM_FOLDERSUB, SC_MARK_EMPTY )
		markerDefine( SC_MARKNUM_FOLDERTAIL, SC_MARK_EMPTY )
		markerDefine( SC_MARKNUM_FOLDEREND, SC_MARK_EMPTY )
		markerDefine( SC_MARKNUM_FOLDEROPENMID, SC_MARK_EMPTY )
		markerDefine( SC_MARKNUM_FOLDERMIDTAIL, SC_MARK_EMPTY )

		@bookmarkMarker = 0
		@bookmarkMarkerMask = 1  << @bookmarkMarker
		markerDefine( @bookmarkMarker, SC_MARK_CIRCLE )
		markerSetFore( @bookmarkMarker, Teal )
		markerSetBack( @bookmarkMarker, Aqua )

#    markerDefine( 7, SC_MARK_SHORTARROW )
#	markerAdd( 0, 7 )

		self.setFoldFlags(16)
		
		self.caretWidth = 2

    self.undoCollection = true

  end

  def setStyle( style, fore, back = White, size = nil, face = nil )
    styleSetFore( style, fore )
    styleSetBack( style, back )
		styleSetSize( style, size ) if size
		styleSetFont( style, face ) if face
  end

  def setUpIndent()
    self.tabWidth = 2
    self.indent = tabWidth()
    self.backSpaceUnIndents = true
  end

  def eventCharAdded( ch )
    if ch == effectiveEolCharacter()
TraceDebug.puts "-------------------------------------------------"
#TraceDebug.puts "eventCharAdded: " << ch.inspect() << ", carret at: "  << currentPos().to_s() << " = "  << charAt( currentPos() ).chr().inspect()
			smartIndent()
		end
  end

  def effectiveEolCharacter()
    return "\n" if eOLMode() == SC_EOL_CRLF || eOLMode() == SC_EOL_LF
		return "\r"
  end		
	
  # changes the indentation of the previous and current lines
  def smartIndent()
    currentLine = currentLineNumber()
#TraceDebug.puts "Current line: "  << currentLine.to_s()
		if currentLine > 0
TraceDebug.puts "Testing previous line"
		  previousLineNumber = currentLine-1
			previousLineKeyWord = firstWordThatIsKeyWordAndNoEndOnLine( previousLineNumber )
			if shouldUnindentLine( previousLineNumber )
        if shouldIndentLine( previousLineNumber )
TraceDebug.puts "Copying previous previous line indentation"
          changeLineIndentation( previousLineNumber, 0, false )       
        else
TraceDebug.puts "Unindenting previous line"
			    unindentLine( previousLineNumber, false )
        end
			elsif shouldIndentLine( previousLineNumber )
TraceDebug.puts "Indenting previous line"
			  indentLine( previousLineNumber, false )
			end
		end
		
		if shouldIndentLine( currentLine )
TraceDebug.puts "Indenting current line"
			indentLine( currentLine, true )
		else
TraceDebug.puts "Copying previous line indentation for current line"
			changeLineIndentation( currentLine, 0, true )
		end
		
#TraceDebug.puts "Carret at: "  << currentPos().to_s() << " = "  << charAt( currentPos() ).chr().inspect()
  end

  # indicates that the line should be unindented if it contains either 
	# 'end' or 'else' and no other key words.
  def shouldUnindentLine( lineNumber )
		firstKeyWord = firstWordThatIsKeyWordAndNoEndOnLine( lineNumber )
		%w( end else ).include?( firstKeyWord )
  end

	# indicates that the line should be indented if the previous line starts with 
	# 'class, def, for...' and does not contains 'end'
  def shouldIndentLine( lineNumber )
		previousNonBlankLine = findNonBlankLineBefore( lineNumber )
		firstKeyWord = firstWordThatIsKeyWordAndNoEndOnLine( previousNonBlankLine )
		%w( module class def for while do if else ).include?( firstKeyWord )
  end
  
  # decrease the indentation level by one in comparison to the previous non blank line
  def unindentLine( lineNumber, updateCaretPosition )
    changeLineIndentation( lineNumber, -1, updateCaretPosition )
  end
  
  # increase the indentation level by one in comparison to the previous non blank line
  def indentLine( lineNumber, updateCaretPosition )
    changeLineIndentation( lineNumber, 1, updateCaretPosition )
  end

  # returns the indentation of the first non blank line found before lineNumber. 0 if none is found.  
  def previousNonBlankLineIndentation( lineNumber )
    while lineNumber > 0
      lineNumber -= 1
			return lineIndentation( lineNumber ) if !blankLine?( lineNumber )
    end
    return 0
  end
	
	# returns the line number of the first line before lineNumber which is not a blank line, -1 if none
	def findNonBlankLineBefore( lineNumber )
		while lineNumber >= 0
			lineNumber -= 1
			return lineNumber if !blankLine?( lineNumber )
		end
		return -1
	end
  
  def blankLine?( lineNumber )
    return getLine( lineNumber ) =~ /^\s*$/
  end
  
  # increase, decrease  the indentation level by one in comparison to the previous non blank line,
  # or copy the indentation level
  # direction: 0=> copy indentation level, 1=> increase, -1=> decrease
  # updateCaretPosition: if true, then the caret to moved to the end of the indentation
  def changeLineIndentation( lineNumber, direction, updateCaretPosition )
		indentation = [0, previousNonBlankLineIndentation(lineNumber) + direction*indent() ].max()
		setLineIndentation( lineNumber, indentation )
		gotoPos( lineIndentPosition( lineNumber ) ) if updateCaretPosition
  end

  def firstWordThatIsKeyWordAndNoEndOnLine( lineNumber )
	  keyWords = lineKeyWords( lineNumber )
	  if !keyWords.empty?
			keyWord = keyWords[0]
			keyWordIndex = keyWord[1]
			if getLine( lineNumber )[0..keyWordIndex].chop() =~ /^\s*$/
				for keyWord in keyWords[1..-1]
				  if keyWord[0] == 'end'
				    return ''
					end
				end
				return keyWord[0]
			end
		end
		return ''
  end

  # Retreives the keywords of the specified line
  def lineKeyWords( lineNumber )
	  keyWords = Array.new
    return keyWords if lineNumber < 0
	  styledText = styledText( positionFromLine( lineNumber), positionFromLine( lineNumber+1 ) )
#TraceDebug.puts "Styled text: " << styledText.inspect()
	  text = styledText[0]
	  styles = styledText[1]
#TraceDebug.puts "Text: " << text.inspect()
#TraceDebug.puts "Styles: " << styles.inspect()
	  startingIndex = 0
		while startingIndex < styles.length()
		  index = styles[startingIndex..-1].index( SCE_P_WORD )
		  if index == nil
		    break
			end
      index += startingIndex
			endIndex = styles[index..-1].indexNotOf( SCE_P_WORD )
      endIndex += index
			keyWords.push( Array.new  << text[index..endIndex-1] << index )
#TraceDebug.puts "Adding keyword: <"  <<  keyWords.last()[0]  << ">, at:"  << index.to_s() << ', '  << endIndex.to_s()
			startingIndex = endIndex
		end
		keyWords
  end

  def eventUpdateUI
    highlightBraces()
  end

  def highlightBraces()
    braceFound = false
    currentPosition = currentPos();
		if currentPosition < textLength()
			braceFound = braceFound || tryHighlightBraceAt( currentPosition, '([{' )
		end
		if currentPosition > 0
			braceFound = braceFound || tryHighlightBraceAt( currentPosition-1, ')]}' )
		end
		hideBracesHighlight() unless braceFound
  end

  def tryHighlightBraceAt( position, braces )
    character = textRange( position, position+1 )
											#TraceDebug.puts "Testing char at position:"  <<  position.to_s() <<  ", char is: " << character
		if braces.rindex( character ) == nil  or !operatorStyle?( styleAt(position) )
			return false
		end

    oppositeBracePosition = braceMatch( position )
    if oppositeBracePosition == -1
											#TraceDebug.puts "Matching brace not found: "  <<  character
			braceBadLight( position )
		else
												#TraceDebug.puts "Highlighting braces: "  <<  character << '(' << position.to_s()  << ','  <<  oppositeBracePosition.to_s()  << ')'
			braceHighlight( position, oppositeBracePosition )
		end
		true
  end

  def hideBracesHighlight
    braceHighlight( INVALID_POSITION, INVALID_POSITION )
  end

  def operatorStyle?( style )
    style == SCE_P_OPERATOR
  end

  def eventMarginClick( modifiers, position, margin )
    if margin == @folderMargin  and  currentLineNumber() > 0
TraceDebug.puts "Toggling fold on line: "  << currentLineNumber().to_s()
			toggleFold( currentLineNumber() )
		end
  end

  # load a new file in the edit control
  def loadFile( fileName )
    self.fileName = fileName
    clearAll()
		text = ''
    File.open( fileName, 'r' ) do |file|
      file.each_line do |line|
				text << line
			end
		end
		addText( text )
		documentStart()
		emptyUndoBuffer()
		setSavePoint()
    true
  end

  def saveFile()
    File.open( fileName(), 'w' ) do |file|
			for lineNumber in 0...lineCount()
				line = self.getLine( lineNumber )
				file.puts line.gsub( /\r|\n/, '' )
			end
		end
		setSavePoint()
		true
  end

# extension:
# def doTitle=( title )

  def title=( title )
    @title = title
		self.doSetTitle( title )
  end

  def fileName=( fileName )
    @fileName = fileName
    baseName = File.basename( fileName.gsub( /\\/, '/') )
		path = File.dirname( fileName.gsub( /\\/, '/') )
		self.title = baseName << ' -- '  << path
  end

  def eventSavePointReached
		@dirty = false
  end

  def eventSavePointLeft
		@dirty = true
  end
	
	def dirty?
		@dirty
	end
	
  def bookmarkToggleOnCurrentLine()
    bookmark = bookmarkAt( currentLineNumber() )
    if bookmark
      markerDeleteHandle( bookmark )
      @bookmarks.delete( bookmark )
    else
      @bookmarks.push( markerAdd( currentLineNumber(), @bookmarkMarker ) )
    end
  end

  def bookmarkAt( line )
    for bookmark in @bookmarks
      if markerLineFromHandle( bookmark ) == line
        return bookmark
      end
    end
    return nil
  end

  def bookmarkMoveToNext()
    line = markerNext( currentLineNumber()+1, @bookmarkMarkerMask )
		line = markerNext( 0, @bookmarkMarkerMask ) if line < 0
    moveToLine( line )
  end

  def bookmarkMoveToPrevious()
    line = 0
    line = markerPrevious( currentLineNumber()-1, @bookmarkMarkerMask ) if currentLineNumber() >0
		line = markerPrevious( lineCount(), @bookmarkMarkerMask ) if line < 0
    moveToLine( line )
  end

  def bookmarkMoveToLast()
    line = markerLineFromHandle( @bookmarks.last() )
    moveToLine( line )
  end

  def bookmarks?()
    @bookmarks.empty?()
  end

  def moveToLine( line )
    gotoLine( line )
  end

  def foldAll()
		TraceDebug.puts "Folding all folders"
		
  end
end




