
require 'TraceDebug'
require 'EditControl'
require 'FileDialog'
require 'Dialog'

class ActionStatusUpdater
# def enable( enabled )
end

# should probably be a module,mixin or whatever (need read doc)
class Action
  def execute()
  end

  def updateStatus( updater )
    updater.enable = enabled?()
  end

  def enabled?()
    true
  end
end


class MainFrameAction < Action
  attr_reader :mainFrame

  def initialize( mainFrame )
    @mainFrame = mainFrame
  end
end



class EditorAction < MainFrameAction
  def initialize( mainFrame )
    super( mainFrame )
  end

  def editor()
    mainFrame().activeWindow()
  end

  # only called if enabled? returns true
  def execute()
    executeEditorAction()
  end

  # called with a valid editor
  def executeEditorAction()
  end

  def enabled?()
    super()  &&  editor() != nil
  end
end


class SimpleEditorAction < EditorAction
  def initialize( mainFrame, executeMethod, enabledMethod =nil )
    super( mainFrame )
		@executeMethod = executeMethod
		@enabledMethod = enabledMethod
  end

  def enabled?()
    super()  &&  ( @enabledMethod ? editor().send( @enabledMethod ) : true )
  end

  def executeEditorAction()
    editor().send( @executeMethod )
  end
end


class FileNewEditorAction < MainFrameAction
  def initialize( mainFrame )
    super
  end

  def execute()
    newEditor = mainFrame.newEditorWindow()
	  newEditor.title = mainFrame().nextUntitledEditorDocumentTitle()
  end
end


class FileOpenEditorAction < MainFrameAction
  def initialize( mainFrame )
    super
  end

  def execute()
    fileNames = FileDialog.openMultiSelect( [ [ 'Ruby Files (*.rb)', '*.rb' ],
																						  [ 'All Files (*.*)', '*.*' ] ],
																						'*.rb',
																						'',
																						[:mustExist] )
    if fileNames
			for fileName in fileNames
				TraceDebug.puts "FileOpen: %s" % fileName
				newEditor = mainFrame().newEditorWindow()
				newEditor.loadFile( fileName ) if newEditor
			end
		end
  end
end


class FileSaveEditorAction < EditorAction
  def initialize( mainFrame, askFileName )
    super( mainFrame )
		@askFileName = askFileName
  end

  def executeEditorAction()
    if !editor().fileName or @askFileName
  	  fileName = FileDialog.save( [ [ 'Ruby Files (*.rb)', '*.rb' ],
																		[ 'All Files (*.*)', '*.*' ] ],
																	'*.rb',
																	'',
																	[:overwritePrompt] )
			if !fileName
				return nil
			end
			editor().fileName = fileName if fileName
		end
		TraceDebug.puts "Saving file as: "  <<  editor.fileName() 
		editor().saveFile()
  end
end

class FindEditorAction < EditorAction
  def initialize( mainFrame, executeMethod, enabledMethod = nil )
    super( mainFrame )
		@executeMethod = executeMethod
		@enabledMethod = enabledMethod
  end

  def executeEditorAction()
    self.send( @executeMethod )
	end

  def enabled?()
    enable = super()
		enable = enable && self.send( @enabledMethod ) if @enabledMethod
		enable
  end
	
	def finder()
		mainFrame().finder()
	end
	
	def hasFinder?()
		finder() != nil
	end
			
	def findWhat()
		findWhat = Dialog.find( finder() ? finder().findWhat() : '' )
		if findWhat
			mainFrame().finder = SimpleFinder.new( findWhat )
			findNext( false )
		else
			mainFrame().finder = nil
		end
	end
	
	def findNext( skipCurrent =true )
		TraceDebug.puts "Find next...." + ( finder() ? finder().inspect() : 'nil' )
		editor().gotoPos( editor().currentPos() +1 ) if skipCurrent
		editor().searchAnchor()
		if editor().searchNext( 0, finder().findWhat() ) == -1
			messageSearchFailed()
		else
		TraceDebug.puts "Find next....found" + ( finder() ? finder().inspect() : 'nil' )
			editor().scrollCaret()
		end
	end
	
	def findPrevious( skipCurrent =true )
		editor().gotoPos( editor().currentPos() -1 ) if skipCurrent  &&  editor().currentPos() > 0
		editor().searchAnchor()
		if editor().searchPrev( 0, finder().findWhat() ) == -1
			messageSearchFailed()
		else
			editor().scrollCaret()
		end
	end
	
	def messageSearchFailed()
		Dialog.message( 'Could not find: ' + finder().findWhat() )
	end
	
end



class SimpleFinder
	attr_accessor :findWhat
	
	def initialize( findWhat )
	  @findWhat = findWhat
	end
end


class MainFrame

  attr_reader :activeWindow
  attr_accessor :finder

  def initialize()
    @actions = Hash.new
    @activeWindow = nil
		@untitledEditorDocumentSerial = 1
		@finder = nil
		@editors = Array.new

		registerDefaultActions()
  end

  def activeWindow=( window )
    @activeWindow = window
		TraceDebug.puts "Activated window <%s>" % window.inspect()
  end

  def actionFor( actionId )
    @actions[actionId]
  end

  def registerAction( actionId, action )
    @actions[actionId] = action
  end

  def unregisterAction( actionId )
    @actions.delete(actionId)
  end

	# hardcoded for now...

	ID_FILE_NEW      =0xE100
	ID_FILE_OPEN     =0xE101
	ID_FILE_CLOSE    =0xE102
	ID_FILE_SAVE     =0xE103
	ID_FILE_SAVE_AS  =0xE104
	ID_EDIT_COPY     =0xE122
	ID_EDIT_CUT      =0xE123
	ID_EDIT_PASTE    =0xE125
	ID_EDIT_UNDO     =0xE12B
	ID_EDIT_REDO     =0xE12C

	ID_BOOKMARK_TOGGLE   =40000
	ID_BOOKMARK_PREVIOUS =40001
	ID_BOOKMARK_NEXT     =40002
	ID_BOOKMARK_LAST     =40003
	ID_FOLD_ALL  =40004
	ID_FIND = 40005
	ID_FIND_NEXT = 40006
	ID_FIND_PREVIOUS = 40007


  def registerDefaultActions()
    registerAction( ID_EDIT_COPY, SimpleEditorAction.new( self, :copy ) )
    registerAction( ID_EDIT_CUT, SimpleEditorAction.new( self, :cut ) )
    registerAction( ID_EDIT_PASTE, SimpleEditorAction.new( self, :paste, :canPaste? ) )
    registerAction( ID_EDIT_UNDO, SimpleEditorAction.new( self, :undo, :canUndo? ) )
    registerAction( ID_EDIT_REDO, SimpleEditorAction.new( self, :redo, :canRedo? ) )

    registerAction( ID_FILE_OPEN, FileOpenEditorAction.new( self ) )
		registerAction( ID_FILE_NEW, FileNewEditorAction.new( self ) )
		registerAction( ID_FILE_SAVE, FileSaveEditorAction.new( self, false ) )
		registerAction( ID_FILE_SAVE_AS, FileSaveEditorAction.new( self, true ) )
	
		registerAction( ID_BOOKMARK_TOGGLE, SimpleEditorAction.new( self, :bookmarkToggleOnCurrentLine ) )
		registerAction( ID_BOOKMARK_NEXT, SimpleEditorAction.new( self, :bookmarkMoveToNext, :bookmarks? ) )
		registerAction( ID_BOOKMARK_PREVIOUS, SimpleEditorAction.new( self, :bookmarkMoveToPrevious, :bookmarks? ) )
		registerAction( ID_BOOKMARK_LAST, SimpleEditorAction.new( self, :bookmarkMoveToLast, :bookmarks? ) )
		registerAction( ID_FOLD_ALL, SimpleEditorAction.new( self, :foldAll ) )
		
		registerAction( ID_FIND, FindEditorAction.new( self, :findWhat ) )
		registerAction( ID_FIND_NEXT, FindEditorAction.new( self, :findNext, :hasFinder? ) )
		registerAction( ID_FIND_PREVIOUS, FindEditorAction.new( self, :findPrevious, :hasFinder? ) )
  end


  def nextUntitledEditorDocumentTitle
  	title = "RubySource%d" % @untitledEditorDocumentSerial
    @untitledEditorDocumentSerial += 1
		title
  end


  def newEditorWindow()
    editor = doNewEditorWindow()
		@editors <<  editor
		editor
  end
	
	def removeEditorWindow( editor )
		@editors.delete!( editor )
		@activeWindow = nil if @activeWindow.equal?( editor )
	end

	def hasDirtyEditor?()
		@editors.each do |editor| 
      return true if editor.dirty?() 
    end
		false
	end

# extension:
#  Returns an EditorControl if success, nil otherwise
#  def doNewEditorWindow()
end








