# Base class for generated class ScintillaBridge
# handle type conversion and is a ruby extension

class Color
  attr_reader :rgb

  # rgb: hexa color:  0x00RRGGBB
  def initialize( rgb )
    setRgb( (rgb & 0xff0000) >> 16, (rgb & 0xff00) >> 8, rgb & 255 )
  end

  def setRgb( red, green, blue )
    @rgb = ((blue & 255) << 16) + ((green & 255) << 8) + (red & 255)
  end

  def blue()
    (@rgb >> 16) & 255
  end

  def green()
    (@gb >> 8) & 255
  end

  def red()
    @rgb& 255
  end

  def blue=( b )
    @rgb = (@rgb & 0xff0000) | ((b&255) << 16)
  end

  def green=( g )
    @rgb = (@rgb & 0xff00) | ((g&255) << 8)
  end

  def blue=( r )
    @rgb = (@rgb & 0xff) | (r&255)
  end

  # predefined color
  Black  = Color.new( 0x000000 )
  Silver = Color.new( 0xC0C0C0 )
  Gray   = Color.new( 0x808080 )
  White  = Color.new( 0xFFFFFF )
  Maroon = Color.new( 0x800000 )
  Red    = Color.new( 0xFF0000 )
  Purple = Color.new( 0x800080 )
  Fuchsia= Color.new( 0xFF00FF )
  Green  = Color.new( 0x008000 )
  Lime   = Color.new( 0x00FF00 )
  Olive  = Color.new( 0x808000 )
  Yellow = Color.new( 0xFFFF00 )
  Navy   = Color.new( 0x000080 )
  Blue   = Color.new( 0x0000FF )
  Teal   = Color.new( 0x008080 )
  Aqua   = Color.new( 0x00FFFF )
  AquaBlue = Color.new( 0x0080C0 )

end

class ScintillaBase

  def initialize( editorWindowHandle )
    if not editorWindowHandle.kind_of?( Integer )
      raise ArgumentError, "#{editorWindowHandle} is not a window handle (Integer)"
	end
    @editorWindowHandle = editorWindowHandle
  end

  def asInt( value )
    raise ArgumentError, "#{value} is not a numeric" unless value.kind_of?( Numeric )
    value.to_i()
  end

  def asBoolean( value )
    value ? true : false
  end

  def asString( value )
    value.to_s()
  end

  def asColor( value )
    raise ArgumentError, "#{value} is not a Color" unless value.kind_of?( Color )
	value.rgb()
  end

  # Function added by the extension:
  # def sendEditorMessage( code, wParam, lParam )
  # => send a message to the editor window, and returns result value.

# Event are received through object of class 'ScintillaNotification'.
# A reader accessor is available for each field. It has the same name
# the C structure field name:
#
# notification.eventType() returns the type of the event (SCN_XXX)
# notification.ch() return a string containing the character ch (length=1)
#
#
# WARNING: only the field compatible with the event type should be accessed!!!
#
# struct SCNotification {
# 	struct NotifyHeader nmhdr;
# 	int position;	// SCN_STYLENEEDED, SCN_MODIFIED, SCN_DWELLSTART, SCN_DWELLEND
# 	int ch;		// SCN_CHARADDED, SCN_KEY
# 	int modifiers;	// SCN_KEY
# 	int modificationType;	// SCN_MODIFIED
# 	const char *text;	// SCN_MODIFIED
# 	int length;		// SCN_MODIFIED
# 	int linesAdded;	// SCN_MODIFIED
# 	int message;	// SCN_MACRORECORD
# 	uptr_t wParam;	// SCN_MACRORECORD
# 	sptr_t lParam;	// SCN_MACRORECORD
# 	int line;		// SCN_MODIFIED
# 	int foldLevelNow;	// SCN_MODIFIED
# 	int foldLevelPrev;	// SCN_MODIFIED
# 	int margin;		// SCN_MARGINCLICK
# 	int listType;	// SCN_USERLISTSELECTION
# 	int x;			// SCN_DWELLSTART, SCN_DWELLEND
# 	int y;		// SCN_DWELLSTART, SCN_DWELLEND
# };
  def scintillaEvent( notification )
  end


  # Returns the current line number
  def currentLineNumber
    lineFromPosition( currentPos() )
  end

  # Adds a text at the current position
  def addText( text )
    addTextMessage( text.length, text )
  end

  def replaceTarget( text )
    replaceTargetMessage( text.length(), text )
  end
  
  def replaceTextRange( from, to, text )
    self.targetStart = from
    self.targetEnd = to
    replaceTarget( text )
  end

#fun int GetTextRange=2162(, textrange tr)
# Retrieve a range of text.
# Returns text including position from, to position to excluded.
  def textRange( from, to )
    sendEditorMessage( 2162, asInt(from), asInt(to) )
  end

#fun void AssignCmdKey=2070(keymod km, int msg)
# When key+modifier combination km is pressed perform msg.
  def assignCommandKey( key, modifiers, command )
    sendEditorMessage( 2070, makeKeyModifiers(key,modifiers), command )
  end

#fun void ClearCmdKey=2071(keymod km,)
# When key+modifier combination km do nothing.
  def clearCommandKey( key, modifiers )
    sendEditorMessage( 2071, makeKeyModifiers(key,modifiers), nil )
  end

  def makeKeyModifiers( key, modifiers )
    (asInt(key) & 0xffff) | ((asInt(modifiers) & 0xffff)<<16)
  end

# Retrieve a range of text and its associated styles.
# Returns a pair [text, [styles]]. First element contains a string with
# the contains in the specified range (from,to). The second element is
# an array of Integer, indicating the style of each character of text.
#fun int GetStyledText=2015(, textrange tr)
  def styledText( from, to )
    sendEditorMessage( 2015, asInt( from ), asInt( to ) )
  end

end

